//
// QPerforce is a gui interface to the perforce revision control
// system. It is based on the qt library, and should be easily portable
// across any platform that has this toolkit available.
// 
// Copyright (C) 2002 Jacob Gladish
// 
// TODO: find legit copyright.
//
#include "ChangeListMenu.h"
#include "P4Process.h"
#include "SubmittedChangeList.h"
#include "todo.h"
#include "WidgetId.h"

#include <qbuttongroup.h>
#include <qradiobutton.h>

#define ADD_BUTTON_TO_GROUP( B, G, POPUP )\
  G->insert(B);\
  POPUP->insertItem(B);

ChangeListMenu::ChangeListMenu( QWidget* parent, const char* name ) :
  QPopupMenu( parent, name )
{
  insertItem( "New...", this, SLOT(slotNew()) );
  insertItem( "Edit Spec...", this, SLOT(slotEditSpec()) );
  insertItem( "Describe...", this, SLOT(slotDescribe()) );
  insertItem( "Describe Changelist...", this, SLOT(slotDescribeChangeList()) );
  insertItem( "Delete", this, SLOT(slotDelete()) );
  insertItem( "Submit...", this, SLOT(slotSubmit()) );

  insertSeparator();

  insertItem( "Add Job Fix...", this, SLOT(slotAddJobFix()) );
  insertItem( "Describe Job Fix...", this, SLOT(slotDescribeJobFix()) );
  insertItem( "Remove Job Fix", this, SLOT(slotRemoveJobFix()) );
  insertItem( "Revert Unchanged Files", this, SLOT(slotRevertUnchangedFiles()) );
  insertItem( "Sync to Changelist...", this, SLOT(slotSyncToChangeList()) );

  // TODO: make sub-men
  insertItem( "Integrate", this, SLOT(slotIntegrate()) );

  insertSeparator();

  insertItem( "Sort Files in Pending Changelists by Extension", this, 
    SLOT(slotSortFilesInPendingChangelistsByExtension()) );
  insertItem( "Sort Changes by User", this, SLOT(slotSortChangesByUser()) );

  insertSeparator();

  //======================================================================
  // filter radio buttons
  //
  
  _buttonGroup = new QButtonGroup();

  QObject::connect( _buttonGroup, SIGNAL(clicked(int)), 
                    this, SLOT(slotFilterChangelists(int)) );

  _unfilteredSubmittedChangelists = new QRadioButton(
    "Unfiltered Submitted Changelists", _buttonGroup );
  ADD_BUTTON_TO_GROUP( _unfilteredSubmittedChangelists, _buttonGroup, this );

  _fscForSelectedFiles = new QRadioButton(
    "Filtered Submitted Changelists for Selected Files", _buttonGroup );
  ADD_BUTTON_TO_GROUP( _fscForSelectedFiles, _buttonGroup, this );
  

  _fscForSelectedFilesIncludingIntegrations = new QRadioButton(
    "Filtered Submitted Changelists for Selected Files, including Integrations",
    _buttonGroup );
  ADD_BUTTON_TO_GROUP( _fscForSelectedFilesIncludingIntegrations, 
    _buttonGroup, this );  

  _fscByClient = new QRadioButton( "Filtered Submitted Changelists by Client...",
    _buttonGroup );
  ADD_BUTTON_TO_GROUP( _fscByClient, _buttonGroup, this );
  

  _fscByUser = new QRadioButton( "Filtered Submitted Changelists by User...",
    _buttonGroup );
  ADD_BUTTON_TO_GROUP( _fscByUser, _buttonGroup, this );    
  
  //======================================================================

  insertSeparator();

  insertItem( "View Pending Changelists", this, 
    SLOT(slotViewPendingChangelists()) );
  insertItem( "View Submitted Changelists", this,
    SLOT(slotViewSubmittedChangelists()) );
}

ChangeListMenu::~ChangeListMenu()
{
  
}

void ChangeListMenu::slotNew()
{
  TODO();
}

void ChangeListMenu::slotEditSpec()
{
  TODO();
}

void ChangeListMenu::slotDescribe()
{
  TODO();
}

void ChangeListMenu::slotDescribeChangeList()
{
  TODO();
}

void ChangeListMenu::slotDelete()
{
  TODO();
}

void ChangeListMenu::slotSubmit()
{
  TODO();
}

void ChangeListMenu::slotAddJobFix()
{
  TODO();
}

void ChangeListMenu::slotDescribeJobFix()
{
  TODO();
}

void ChangeListMenu::slotRemoveJobFix()
{
  TODO();
}

void ChangeListMenu::slotRevertUnchangedFiles()
{
  TODO();
}

void ChangeListMenu::slotSyncToChangeList()
{
  TODO();
}

void ChangeListMenu::slotIntegrate()
{
  TODO();
}

void ChangeListMenu::slotSortFilesInPendingChangelistsByExtension()
{
  TODO();
}

void ChangeListMenu::slotSortChangesByUser()
{
  TODO();
}

void ChangeListMenu::slotFilterChangelists( int n )
{
  TODO();
}

void ChangeListMenu::slotViewPendingChangelists()
{
  TODO();
}

void ChangeListMenu::slotViewSubmittedChangelists()
{  
  ASSERT( _submittedChangeListsTable != NULL );
  _submittedChangeListCache.updateCacheIfNecessary();
  _submittedChangeListsTable->clear();

  Perforce::SubmittedChangeList* c;
  Perforce::SubmittedChangeListList changes = 
    _submittedChangeListCache.getCache();

  for (c = changes.first(); c != NULL; c = changes.next()) {
    _submittedChangeListsTable->addSubmittedChangeList( c );
  }

  emit showSubmittedChangeLists( SUBMITTED_CHANGELISTS_TABLE );
}

void ChangeListMenu::setSubmittedChangeListsView( SubmittedChangeListsTable* t )
{
  ASSERT( t != NULL );
  this->_submittedChangeListsTable = t;
}

ChangeListMenu::SubmittedChangeListCache::SubmittedChangeListCache() 
{
  
}

void ChangeListMenu::SubmittedChangeListCache::updateCache()
{
  P4Process p4;
  _cache = p4.getSubmittedChangeLists();
}
